package com.rshy.project.hy.manager.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.IdcardUtil;
import cn.hutool.core.util.StrUtil;
import com.rshy.project.hy.manager.RshyJkManager;
import com.rshy.project.hy.model.constant.ItfwConstant;
import com.rshy.project.hy.model.dto.NucleicAcidDTO;
import com.rshy.project.hy.model.enums.*;
import com.rshy.project.hy.server.ItfwServer;
import com.rshy.project.hy.server.VIIDServer;
import com.rshy.project.hy.server.YituFaceServer;
import com.rshy.project.hy.server.dto.*;
import com.rshy.project.hy.server.param.RshyParam;
import com.rshy.project.hy.server.param.RshyQuickParam;
import com.rshy.project.hy.server.vo.RetrievaResultVO;
import com.rshy.project.hy.server.vo.RetrievalRepositoryVO;
import com.rshy.project.hy.server.vo.RshyVo;
import com.rshy.project.hy.util.ObjectUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;

/**
 * @author: lileilei
 * @date: 2022/4/23 20:12
 * @description:
 */
@Component
@Slf4j
public class RshyJkManagerImpl implements RshyJkManager {
    @Autowired
    private YituFaceServer yituFaceServer;
    @Autowired
    private ItfwServer itfwServer;
    @Autowired
    private VIIDServer viidServer;
    @Autowired
    private StringRedisTemplate stringRedisTemplate;

    @Override
    public RshyVo getRshyInfo(RshyParam rshyParam) {
        // timeout 转成毫秒
        rshyParam.setTimeout(rshyParam.getTimeout() * 1000 - 800);
        RshyVo rshyVo = new RshyVo();
        PctgkryDTO gkryDTO = new PctgkryDTO();
        ResultDTO resultDTO = new ResultDTO();
        resultDTO.setPctgkryDTO(gkryDTO);
        DateTime beginDate = DateTime.now();
        PctjkmInfoDTO jkmInfo = null;

        if (StringUtils.isNotBlank(rshyParam.getBarCode()) || StringUtils.isNotBlank(rshyParam.getSfzh())) {
            try {
                //查询健康码信息
                jkmInfo = itfwServer.getPctjkm(rshyParam.getBarCode(), rshyParam.getSfzh(), rshyParam.getTimeout());
            } catch (Exception e) {
                log.error("查询健康码信息异常:{}", e.getMessage());
                e.printStackTrace();
            }

            if (jkmInfo == null) {
                return rshyVo.setPassStatus(PassStatusEnum.NO_CODE);
            }

            rshyParam.setSfzh(jkmInfo.getId_num());
            rshyParam.setXm(jkmInfo.getUser_name());
            String codeLabel = jkmInfo.getCOLOR_CODE_LABEL();
            String phone = jkmInfo.getPn();
            rshyVo.setXm(rshyParam.getXm()).setSfzh(rshyParam.getSfzh()).setHealthCode(codeLabel).setPhone(phone);
        }

        rshyParam.setTimeout((int) (rshyParam.getTimeout() - DateUtil.between(beginDate, DateTime.now(), DateUnit.MS)));

        AtomicReference<RshyVo> quickRshyVo = new AtomicReference<>(new RshyVo());
        List<Runnable> taskList = new ArrayList<Runnable>() {
            {
                add(() -> resultDTO.setPctymDataDTOS(itfwServer.getPctym(rshyParam.getSfzh(), rshyParam.getTimeout())));
                add(() -> resultDTO.setPcthsResultDTO(itfwServer.getPcths(rshyParam.getSfzh(), rshyParam.getTimeout())));
                add(() -> resultDTO.setPctzjzpResultDTO(itfwServer.getPctzjzp(rshyParam.getSfzh(), rshyParam.getTimeout())));

                if (StrUtil.isNotBlank(rshyParam.getPhoto())) {
                    //add(() -> resultDTO.setPersonIdentifyDTO(viidServer.personIdentify(rshyParam.getPhoto(), rshyParam.getTimeout())));
                    add(() -> quickRshyVo.set(getQuickRyhyInfo(new RshyQuickParam().setPhoto(rshyParam.getPhoto()).setTimeout(rshyParam.getTimeout()))));
                }
            }
        };

        //if (rshyParam.getScreenLocked() == 0 && (barCodes == null || PassStatusEnum.GREEN_CODE.getDesc().equals(barCodes.getCOLOR_CODE_LABEL()))) {
        //绿码人员继续进行七大类管控接口和人车核录接口比对
        if (jkmInfo != null && PassStatusEnum.GREEN_CODE.getDesc().equals(jkmInfo.getCOLOR_CODE_LABEL())) {
            taskList.add(() -> resultDTO.getPctgkryDTO().setPctdfkDTO(itfwServer.getPctdfk(rshyParam.getSfzh(), "", rshyParam.getTimeout())));
            taskList.add(() -> resultDTO.getPctgkryDTO().setPctjsryDTO(itfwServer.getPctjsry(rshyParam.getSfzh(), rshyParam.getTimeout())));
            taskList.add(() -> resultDTO.getPctgkryDTO().setPctztDTO(itfwServer.getPctzt(rshyParam.getSfzh(), rshyParam.getTimeout())));
            taskList.add(() -> resultDTO.getPctgkryDTO().setPctzfbaDTO(itfwServer.getPctzfba(rshyParam.getSfzh(), rshyParam.getTimeout())));
            taskList.add(() -> resultDTO.getPctgkryDTO().setPctqkryDTO(itfwServer.getPctqkry(rshyParam.getSfzh(), rshyParam.getTimeout())));
            taskList.add(() -> resultDTO.getPctgkryDTO().setPctxdryDTO(itfwServer.getPctxdry(rshyParam.getSfzh(), "", rshyParam.getTimeout())));
            taskList.add(() -> resultDTO.getPctgkryDTO().setPctzdryDTO(itfwServer.getPctzdry(rshyParam.getSfzh(), rshyParam.getTimeout())));
            taskList.add(() -> resultDTO.setPctrchlDTO(itfwServer.getPctrchl(rshyParam.getSfzh(), rshyParam.getTimeout())));
        }

        //异步流并发处理
        taskList.parallelStream().forEach(v -> {
            try {
                v.run();
            } catch (Exception e) {
                log.error("异常信息：{}", e.getMessage());
                e.printStackTrace();
            }
        });

        log.info("IT服务平台接口调用完成");
        rshyVo.setXm(rshyParam.getXm()).setSfzh(rshyParam.getSfzh()).setTimes(times(rshyParam.getSfzh()));
        //疫苗
        rshyVo.setVaccine(CollectionUtil.isEmpty(resultDTO.getPctymDataDTOS()) ? 0 : resultDTO.getPctymDataDTOS().size());
        //核酸报告
        rshyVo.setNucleic_acid(nucleicAcid(resultDTO.getPcthsResultDTO()));
        //行程卡（暂时没接口，默认0）
        rshyVo.setTravel_card(0);
        rshyVo.setXb(SexEnum.codeOf(IdcardUtil.getGenderByIdCard(rshyParam.getSfzh())).getDesc());
        Date date = DateUtil.parse(IdcardUtil.getBirthByIdCard(rshyParam.getSfzh()));
        rshyVo.setBirthday(DateUtil.format(date, "yyyy-M-d"));

        String hcjg = resultDTO.getPctrchlDTO() == null ? "" : resultDTO.getPctrchlDTO().getHcjg();


        //if (rshyVo.getNucleic_acid().getStatus() == 0) {
        //    rshyVo.setPass(PassStatusEnum.NO_COMPLIANCE.getCode()).setTts(PassStatusEnum.NO_COMPLIANCE.getTts()).setDisplayMsg(PassStatusEnum.NO_COMPLIANCE.getDesc());
        //}else {


        //if (resultDTO.getPersonIdentifyDTO() != null && !rshyParam.getSfzh().equals(resultDTO.getPersonIdentifyDTO().getIdNumber())) {
        //人证或者人码不符
        //  rshyVo.setPassStatus(PassStatusEnum.NO_MATCHED).setWarning(PassStatusEnum.NO_MATCHED.getDesc()).setKind(PassStatusEnum.NO_MATCHED.getDesc());
        //} else
        if (jkmInfo != null && PassStatusEnum.RED_CODE.getDesc().equalsIgnoreCase(jkmInfo.getCOLOR_CODE_LABEL())) {
            //红码
            rshyVo.setPass(PassStatusEnum.RED_CODE.getCode()).setTts(PassStatusEnum.RED_CODE.getTts()).setDisplayMsg(PassStatusEnum.RED_CODE.getDesc()).setWarning(PassStatusEnum.RED_CODE.getDesc()).setKind("健康码" + PassStatusEnum.RED_CODE.getDesc());
        } else if (jkmInfo != null && PassStatusEnum.YELLOW_CODE.getDesc().equalsIgnoreCase(jkmInfo.getCOLOR_CODE_LABEL())) {
            //黄码
            rshyVo.setPassStatus(PassStatusEnum.YELLOW_CODE).setKind("健康码" + PassStatusEnum.YELLOW_CODE.getDesc());
        } else if (resultDTO.getPctgkryDTO().getPctztDTO() != null || StrUtil.containsAnyIgnoreCase(hcjg, "抓捕") || StrUtil.containsAnyIgnoreCase(hcjg, "拦截")) {
            //在逃 或者 人车核录接口返回 抓捕或者拦截
            rshyVo.setPassStatus(PassStatusEnum.NO_PASSING).setKind(kind(resultDTO)).setWarning(warning(resultDTO));//.setWarning("在逃人员:" + resultDTO.getPctgkryDTO().getPctztDTO().getJyaq()).setKind(resultDTO.getPctgkryDTO().getPctztDTO().getZtrylxmc());
            //} else if (ObjectUtils.allFieldIsNULL(resultDTO.getPctgkryDTO()) || StrUtil.containsAnyIgnoreCase(hcjg, "通过")) {
            //    rshyVo.setPass(PassStatusEnum.NORMAL_TRAFFIC.getCode()).setDisplayMsg(PassStatusEnum.NORMAL_TRAFFIC.getDesc()).setTts(PassStatusEnum.NORMAL_TRAFFIC.getTts());
        } else if (!ObjectUtils.allFieldIsNULL(resultDTO.getPctgkryDTO()) || StrUtil.containsAnyIgnoreCase(hcjg, "存疑")) {
            //七大类管控人员(在逃除外)和人车核录接口返回存疑人员
            rshyVo.setPassStatus(PassStatusEnum.RIGHT_WAY).setKind(kind(resultDTO)).setWarning(warning(resultDTO));
        } else if (quickRshyVo.get() != null && StrUtil.isNotEmpty(quickRshyVo.get().getSfzh())) {
            //快速通行
            rshyVo = quickRshyVo.get().setTimes(times(rshyParam.getSfzh()));
        } else {
            //正常通过
            rshyVo.setPassStatus(PassStatusEnum.NORMAL_TRAFFIC);
        }

        if (rshyVo.getPass().equals(PassStatusEnum.NORMAL_TRAFFIC.getCode()) && rshyVo.getHealthCode() != null) {
            rshyVo.setTts(rshyVo.getHealthCode());
        }
        if (resultDTO.getPctzjzpResultDTO() != null) {
            rshyVo.setPhoto(resultDTO.getPctzjzpResultDTO().getPhoto());
        }

        return rshyVo;
    }

    private NucleicAcidDTO nucleicAcid(PcthsResultDTO pcthsResultDTO) {
        NucleicAcidDTO nucleicAcidDTO = new NucleicAcidDTO();
        if (pcthsResultDTO == null) {
            nucleicAcidDTO.setStatus(NucleicAcidEnum.NOT_CONFORM.getCode()).setResults("48h内未采集");
            return nucleicAcidDTO;
        }

        if (pcthsResultDTO.getSHE_RQSJ() != null) {
            DateTime parse = DateUtil.parse(String.valueOf(Integer.valueOf(pcthsResultDTO.getSHE_RQSJ()) * 1000));
            long betweenMs = DateUtil.betweenMs(parse, DateUtil.date());
            //48h防疫规则
            if (betweenMs > 48 * 60 * 60 * 1000) {
                nucleicAcidDTO.setStatus(NucleicAcidEnum.NOT_CONFORM.getCode()).setResults(new StringBuilder("48小时之外已出结果，结果").append(pcthsResultDTO.getJGU_MC()).toString()).setTime(DateUtil.format(parse, "MM-dd HH:mm"));
                return nucleicAcidDTO;
            }

            nucleicAcidDTO.setStatus(NucleicAcidEnum.CONFORM.getCode()).setResults(new StringBuilder("采样48h内 结果").append(pcthsResultDTO.getJGU_MC()).toString()).setTime(DateUtil.format(parse, "MM-dd HH:mm"));
            return nucleicAcidDTO;
        }

        DateTime parse = DateUtil.parse(String.valueOf(Integer.valueOf(pcthsResultDTO.getCJSJ()) * 1000));
        long betweenMs = DateUtil.betweenMs(parse, DateUtil.date());
        //48h防疫规则
        if (betweenMs > 48 * 60 * 60 * 1000) {
            nucleicAcidDTO.setStatus(NucleicAcidEnum.NOT_CONFORM.getCode()).setResults("48小时之外未出结果");
            return nucleicAcidDTO;
        }

        nucleicAcidDTO.setStatus(NucleicAcidEnum.CONFORM.getCode()).setResults("48小时之内未出结果");
        return nucleicAcidDTO;
    }

    @Override
    public RshyVo getQuickRyhyInfo(RshyQuickParam rshyQuickParam) {
        // timeout 转成毫秒
        rshyQuickParam.setTimeout(rshyQuickParam.getTimeout() * 1000);
        RshyVo rshyVo = new RshyVo();
        RetrievaResultVO retrievaResultVO = yituFaceServer.retrieval(rshyQuickParam.getPhoto(), rshyQuickParam.getTimeout());
        List<RetrievalRepositoryVO> voList = Convert.toList(RetrievalRepositoryVO.class, retrievaResultVO.getResults());
        if (CollectionUtil.isEmpty(voList)) {
            return rshyVo.setPass(PassStatusEnum.NORMAL_TRAFFIC.getCode()).setDisplayMsg(PassStatusEnum.NORMAL_TRAFFIC.getDesc()).setTts(PassStatusEnum.NORMAL_TRAFFIC.getTts());
        }

        //查询人像库信息
        RetrievaResultVO repository = yituFaceServer.repository(rshyQuickParam.getTimeout());
        List<RepositoryDTO> dtoList = Convert.toList(RepositoryDTO.class, repository.getResults());
        Map<String, String> stringMap = CollectionUtil.isNotEmpty(dtoList) ? dtoList.stream().collect(Collectors.toMap(RepositoryDTO::getId, RepositoryDTO::getName, (v1, v2) -> v1)) : MapUtil.newHashMap();
        RetrievalRepositoryVO repositoryVO = voList.stream().findFirst().get();
        rshyVo.setPass(RepositoryEnum.COUNTRY_LARGE.getId().equals(repositoryVO.getRepository_id()) ? PassStatusEnum.NO_PASSING.getCode() : PassStatusEnum.RIGHT_WAY.getCode()).setKind(StrUtil.EMPTY)
                .setWarning(stringMap.get(repositoryVO.getRepository_id())).setDisplayMsg(RepositoryEnum.COUNTRY_LARGE.getId().equals(repositoryVO.getRepository_id()) ? PassStatusEnum.NO_PASSING.getDesc() : PassStatusEnum.RIGHT_WAY.getDesc())
                .setTts(RepositoryEnum.COUNTRY_LARGE.getId().equals(repositoryVO.getRepository_id()) ? PassStatusEnum.NO_PASSING.getTts() : PassStatusEnum.RIGHT_WAY.getTts()).setHealthCode(StrUtil.EMPTY).setSfzh(repositoryVO.getPersonId())
                .setXm(repositoryVO.getName()).setXb(SexEnum.codeOf(repositoryVO.getGender()).getDesc()).setMz(StrUtil.EMPTY).setPhone(StrUtil.EMPTY).setAddress(StrUtil.EMPTY);//.setPhoto(rshyQuickParam.getPhoto());
        //log.info("快速核验响应内容：{}", JSON.toJSONString(rshyVo));
        return rshyVo;
    }

    /**
     * 今日进站次数
     *
     * @param idNumber 身份证号
     * @return
     */
    private Integer times(String idNumber) {
        String cacheKey = idNumber + ":" + ItfwConstant.TIMES_KEY;
        int times = 1;
        String timesStr = stringRedisTemplate.opsForValue().get(cacheKey);
        if (timesStr == null) {
            //今日首次进站
            stringRedisTemplate.opsForValue().set(cacheKey, String.valueOf(times), DateUtil.endOfDay(DateUtil.date()).getTime() - DateUtil.date().getTime(), TimeUnit.MILLISECONDS);
        } else {
            times = Integer.parseInt(timesStr) + 1;
            stringRedisTemplate.opsForValue().increment(cacheKey);
        }

        return times;
    }

    /**
     * 人员类型简要概述
     *
     * @param resultDTO
     * @return
     */
    private String kind(ResultDTO resultDTO) {
        StringBuilder stringBuilder = new StringBuilder();
        stringBuilder.append(resultDTO.getPctgkryDTO().getPctztDTO() != null ? resultDTO.getPctgkryDTO().getPctztDTO().getZtrylxmc() + "、" : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctgkryDTO().getPctdfkDTO() != null ? HyTypeEnum.pctdfk.getDesc() + "、" : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctgkryDTO().getPctjsryDTO() != null ? HyTypeEnum.pctjsry.getDesc() + "、" : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctgkryDTO().getPctxdryDTO() != null ? HyTypeEnum.pctxdry.getDesc() + "、" : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctgkryDTO().getPctqkryDTO() != null ? HyTypeEnum.pctqkry.getDesc() + "、" : StrUtil.EMPTY);
        //stringBuilder.append(resultDTO.getPctjkmInfoDTO() != null ? HyTypeEnum.pctjkm.getDesc() : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctgkryDTO().getPctzfbaDTO() != null ? HyTypeEnum.pctzfba.getDesc() + "、" : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctgkryDTO().getPctzdryDTO() != null ? HyTypeEnum.pctzdry.getDesc() + "、" : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctrchlDTO() != null ? resultDTO.getPctrchlDTO().getRylb() : StrUtil.EMPTY);
        return stringBuilder.toString();
    }

    /**
     * 警示信息
     *
     * @param resultDTO
     * @return
     */
    private String warning(ResultDTO resultDTO) {
        StringBuilder stringBuilder = new StringBuilder();
        stringBuilder.append(resultDTO.getPctgkryDTO().getPctztDTO() != null ? resultDTO.getPctgkryDTO().getPctztDTO().getJyaq() + "\n" : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctgkryDTO().getPctdfkDTO() != null ? resultDTO.getPctgkryDTO().getPctdfkDTO().getJyaq() + "\n" : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctgkryDTO().getPctjsryDTO() != null ? resultDTO.getPctgkryDTO().getPctjsryDTO().getJyaq() + "\n" : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctgkryDTO().getPctxdryDTO() != null ? resultDTO.getPctgkryDTO().getPctxdryDTO().getXdcsmc() + "\n" : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctgkryDTO().getPctqkryDTO() != null ? resultDTO.getPctgkryDTO().getPctqkryDTO().getLsqkqk() + "\n" : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctgkryDTO().getPctzfbaDTO() != null ? resultDTO.getPctgkryDTO().getPctzfbaDTO().getAjmc() + "\n" : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctgkryDTO().getPctzdryDTO() != null ? resultDTO.getPctgkryDTO().getPctzdryDTO().getAjlbmc() + "\n" : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctrchlDTO() != null && !resultDTO.getPctrchlDTO().getHcjg().equals("通过") ? resultDTO.getPctrchlDTO().getBjxx() : StrUtil.EMPTY);
        return stringBuilder.toString();
    }
}
